*----+---1---------2---------3---------4---------5---------6---------7-+
      SUBROUTINE PSKINI( ldebug, ifail )
C  Kinematics for the initial-state (spacelike) parton shower
C  The energy-momentum is precisely conserved.
C  The splitting parameter "z" is interpleted as the ratio of the invariant 
C  masses of the collision after and before the branch.
C
C  In this version, "pt" is determined by the relation
C          pt^2 = (1-z) Q^2
C  derived from the massless and collinear approximation which the 
C  LL subtraction is based on.
C  PLPTN(2,i) is now a parameter of the evolution and not equal to the 
C  virtuality of the evolving parton.
C  2006.07.04, S. Odaka
C
C  The return code "ifail" was added.
C  It is =1 if the kinematics determination fails; i.e. E3 < 0.
C  2007.05.09, S. Odaka
C
      IMPLICIT REAL*8(A-H,O-Z)
      include 'inclm.h'
      COMMON/QPLIST/PLPTN(10,10000),NLPTN(10,10000),NTOP

      dimension ipbeam(2),ippart(2),xpart(2)
      dimension bdir(3),ptemp(4)

      if( ldebug.ge.1 ) print *,'----- PSKINI -----'

C  The return code was added on May 8, 2007.
      ifail= 0

C  Find beam particles
      nbeam= 0
      do 10 i=1,NTOP
         if( NLPTN(2,i).eq.0 ) then
            nbeam= nbeam +1
            if( nbeam.le.2 ) then
               ipbeam(nbeam)= i
            endif
         endif
 10   continue
      if( nbeam.ne.2 ) then
         print *,'PSKINI: abnormal number of beam particles.', nbeam
         stop
      endif

C  The system is assumed to be in the CM frame of beam particles, 
C  where the beam momenta are along the z axis.
      ip1= ipbeam(1)
      ip2= ipbeam(2)
      if( PLPTN(4,ip1).ne.0. .or. PLPTN(5,ip1).ne.0. 
     >     .or. PLPTN(4,ip2).ne.0. .or. PLPTN(5,ip2).ne.0. 
     >     .or. PLPTN(6,ip1)+PLPTN(6,ip2).ne.0. ) then
         print *,'PSKINI: the system is not in the CM frame.'
         stop
      endif
      s0= ( PLPTN(7,ip1) +PLPTN(7,ip2) )**2
      if( ldebug.ge.1 ) then
         print '(5X,''beams at'',I3,'' and'',I3,'': s ='',E12.4)',
     >        ip1,ip2,s0
      endif
      if( s0.lt.0. ) then
         print *,'PSKINI: negative s of the beam collision.',s0
         stop
      endif

C  Starting partons
      do 20 i=1,2
         if( NLPTN(3,ipbeam(i)).ne.1 ) then
            print *,'PSKINI: starting parton is not unique.'
            stop
         endif
         ip= ipbeam(i) +NLPTN(4,ipbeam(i))
         ippart(i)= ip
         xpart(i)= PLPTN(1,ip)
 20   continue

C  Basic parameters of the starting system.
      tau1= xpart(1)*xpart(2)
      if( PLPTN(6,ipbeam(1)).gt.PLPTN(6,ipbeam(2)) ) then
         y1= .5d0*log( xpart(1)/xpart(2) )
      else
         y1= .5d0*log( xpart(2)/xpart(1) )
      endif
      s1= s0*tau1
      roots1= sqrt( s1 )
      if( ldebug.ge.1 ) then
         print '('' Starting x = '',2E12.4)', xpart
         print '(5X,''tau, y, s, root(s)  = '',4E14.6)', 
     >        tau1,y1,s1,roots1
      endif

C  Kinematics are tentatively determined in the CM frame of the 
C  starting partons.
      ip1= ippart(1)
      ip2= ippart(2)
CC  The starting partons are assumed to be on-shell.
C      qsq1= ptnmas( NLPTN(1,ip1) )**2
C      qsq2= ptnmas( NLPTN(1,ip2) )**2
C      PLPTN(2,ip1)= qsq1
C      PLPTN(2,ip2)= qsq2
C  The initial virtuality of QCDPS is used as the starting point.
      qsq1= PLPTN(2,ip1)
      qsq2= PLPTN(2,ip2)
      pz= sqrt( ( ( s1 -qsq1 -qsq2 )**2/4.d0 -qsq1*qsq2 )/s1 )
      PLPTN(4,ip1)= 0.d0
      PLPTN(5,ip1)= 0.d0
      if( PLPTN(6,ipbeam(1)).gt.0.d0 ) then
         PLPTN(6,ip1)= pz
      else
         PLPTN(6,ip1)= -pz
      endif
      PLPTN(4,ip2)= 0.d0
      PLPTN(5,ip2)= 0.d0
      if( PLPTN(6,ipbeam(2)).gt.0.d0 ) then
         PLPTN(6,ip2)= pz
      else
         PLPTN(6,ip2)= -pz
      endif
      PLPTN(7,ip1)= ( s1 +qsq1 -qsq2 )/2.d0/roots1
      PLPTN(7,ip2)= ( s1 -qsq1 +qsq2 )/2.d0/roots1
      if( ldebug.ge.1 ) then
         print '(5X,''initial partons'')'
         print '(5X,I3,'': '',4E14.6)', 
     >        ip1,(PLPTN(k,ip1),k=4,7)
         print '(5X,I3,'': '',4E14.6)', 
     >        ip2,(PLPTN(k,ip2),k=4,7)
      endif

C  Loop over branches.
 1    continue

C  Determine which side to branch.
      iside= 0
      q2branch= -1.d15
      do 30 i=1,2
         ipmoth= ippart(i)
C  PS must be spacelike.
         if( NLPTN(6,ipmoth).ne.-1 ) goto 30
         nchild= NLPTN(3,ipmoth)
         if( nchild.gt.0 ) then
            ipch1= ipmoth +NLPTN(4,ipmoth)
C  The virtuality is negative for the evoluting parton, while it is zero
C  for the radiation. 
            q2side= 0.
            do 31 j=1,nchild
               ipch= ipch1 +j -1
               q2= PLPTN(2,ipch)
               if( q2.lt.q2side ) q2side= q2
 31         continue
C  Choose the smaller virtuality side.
            if( q2side.gt.q2branch ) then
               iside= i
               q2branch= q2side
            endif
         endif
 30   continue

C  iside = 0 means that we are already at the end of PS.
      if( iside.lt.1 ) then
         if( ldebug.ge.1 ) then
            print *,'PSKINI: branch completed.'
            print '(5X,''final partons'')'
            print '(5X,I3,'': '',4E14.6)', 
     >           ippart(1),(PLPTN(k,ippart(1)),k=4,7)
            print '(5X,I3,'': '',4E14.6)', 
     >           ippart(2),(PLPTN(k,ippart(2)),k=4,7)
            print '(5X,''final x = '',2E12.4)', xpart
            s= xpart(1)*xpart(2)*s0
            print '(10X,''-> s = '',E14.6)', s
            pxsum= PLPTN(4,ippart(1)) +PLPTN(4,ippart(2))
            pysum= PLPTN(5,ippart(1)) +PLPTN(5,ippart(2))
            pzsum= PLPTN(6,ippart(1)) +PLPTN(6,ippart(2))
            esum= PLPTN(7,ippart(1)) +PLPTN(7,ippart(2))
            ptsum= sqrt( pxsum**2 +pysum**2 )
            s= esum**2 -pxsum**2 -pysum**2 -pzsum**2
            print '(5X,''p, E sum = '',4E14.6)', pxsum,pysum,pzsum,esum
            print '(5X,''s, pt = '',2E14.6)', s,ptsum
         endif
C  Boost the partons to the lab. frame before returning.
C  Boost parameters are determined from the rapidity of the starting 
C  parton system.
         gamma= cosh( y1 )
C         beta= sqrt( 1.d0 -1.d0/gamma**2 )
C         if( y1.lt.0.d0 ) beta= -beta
         beta= tanh( y1 )
         if( ldebug.ge.1 ) then
            print *,'Boost all partons to the lab. frame.'
            print '(5X,''y of the starting parton system = '',
     >           E14.6)', y1
            print '(5X,''beta, gamma = '',2E14.6)', beta,gamma
         endif
         do 91 i=1,ntop
            if( i.eq.ipbeam(1) .or. i.eq.ipbeam(2) ) goto 91
            if( ldebug.ge.3 ) print '(5X,I3,4E14.6)', 
     >           i,(PLPTN(k,i),k=4,7)
            pz= PLPTN(6,i)
            E= PLPTN(7,i)
            PLPTN(6,i)= gamma*( pz +beta*E )
            PLPTN(7,i)= gamma*( beta*pz +E )
            if( ldebug.ge.3 ) print '(5X,'' -> '',4E14.6)', 
     >           (PLPTN(k,i),k=4,7)
 91      continue
         if( ldebug.ge.1 ) then
            print '(5X,''final partons'')'
            print '(10X,I3,'': '',4E14.6)', 
     >           ippart(1),(PLPTN(k,ippart(1)),k=4,7)
            print '(10X,I3,'': '',4E14.6)', 
     >           ippart(2),(PLPTN(k,ippart(2)),k=4,7)
            pxsum= PLPTN(4,ippart(1)) +PLPTN(4,ippart(2))
            pysum= PLPTN(5,ippart(1)) +PLPTN(5,ippart(2))
            pzsum= PLPTN(6,ippart(1)) +PLPTN(6,ippart(2))
            esum= PLPTN(7,ippart(1)) +PLPTN(7,ippart(2))
            s= esum**2 -pxsum**2 -pysum**2 -pzsum**2
            ysum= .5d0*log( ( esum +pzsum )/( esum -pzsum ) )
            ptsum= sqrt( pxsum**2 +pysum**2 )
            print '(5X,''s, y, pt = '',3E14.6)', s,ysum,ptsum
            pxsum= 0.d0
            pysum= 0.d0
            pzsum= 0.d0
            esum= 0.d0
            do 93 i=1,ntop
               if( NLPTN(3,i).lt.1 ) then
                  pxsum= pxsum +PLPTN(4,i)
                  pysum= pysum +PLPTN(5,i)
                  pzsum= pzsum +PLPTN(6,i)
                  esum= esum +PLPTN(7,i)
               endif
 93         continue
            print *,'PSKINI: completed.'
            print '(5X,''total px, py, pz, E = '',4E14.6)', 
     >           pxsum,pysum,pzsum,esum
            ssum= esum**2 -pxsum**2 -pysum**2 -pzsum**2
            ysum= .5d0*log( ( esum +pzsum )/( esum -pzsum ) )
            print '(5X,''s, y = '',3E14.6)', ssum,ysum
         endif
         return
      endif

C  Set the pointers.
      ipmoth= ippart(iside)
      nchild= NLPTN(3,ipmoth)
      if( nchild.ne.2 ) then
         print *,'PSKINI: num. of children is not 2. ',nchild
         stop
      endif
      ipch1= ipmoth +NLPTN(4,ipmoth)
      ipch2= ipch1 +1
      q21= PLPTN(2,ipch1)
      q22= PLPTN(2,ipch2)
      if( q21.lt.q22 ) then
         ipchild= ipch1
         iprad= ipch2
      else
         ipchild= ipch2
         iprad= ipch1
      endif

      if( iside.eq.1 ) then
         iptarget= ippart(2)
      else
         iptarget= ippart(1)
      endif

      if( ldebug.ge.2 ) then
         print '(I3,'' (mother) -> '',I3,'' (child) '',I3,
     >        '' (rad), with '',I3,'' (target): q-sq = '',E12.4)',
     >        ipmoth,ipchild,iprad,iptarget, q2branch
      endif

C  Move to the CM frame of the "mother" and "target".
      call bstcm( PLPTN(4,ipmoth), PLPTN(4,iptarget),
     >     P1, E1, E2, bdir, gamma, phi, theta )

      if( ldebug.ge.2 ) then
         print *,'    PLPTN(1-7,i)'
         print '(10X,''mother: '',7E14.6)', (PLPTN(k,ipmoth),k=1,7)
         print '(10X,''target: '',7E14.6)', (PLPTN(k,iptarget),k=1,7)
         print '(5X,''p1, E1, E2 = '',3E14.6)', P1,E1,E2
         print '(5X,''gamma, phi, theta = '',3E14.6)', gamma,phi,theta
      endif

C  Parameters for the branch.
      zbr= PLPTN(1,ipchild)
      shat= ( E1 +E2 )**2
      shat1= shat*zbr
      qsq1= E1**2 -P1**2
      qsq2= E2**2 -P1**2
C  Radiation is assumed to be on-shell now.
      qsq4= ptnmas( NLPTN(1,iprad) )**2

C  Determine "PT"
      q2abs= -q2branch
      PT2= ( 1.d0 -zbr )*q2abs
CCC      PT2= ( 1.d0 -zbr )**2 * q2abs
CCC      PT2= ( 1.d0 -zbr -q2abs/shat )*q2abs
      if( PT2.lt.0.d0 ) then
         if( ldebug.ge.1 ) print *,'!!! pt-sq < 0 ',
     >        '-> set to zero !!!'
         PT2= 0.d0
      endif

      if( ldebug.ge.2 ) then
         print '(5X,''z, pt-sq, qsq1, qsq2, qsq4 = '',5E14.6)',
     >        zbr, PT2, qsq1,qsq2,qsq4
      endif

C  Solution!

C  E3 becomes negative when the Q-value is not large enough to make
C  the radiation.
C  This may happen when the radiation is massive (e.g., b quark).
C  The event weight should be set to zero in this case.
C  May 8, 2007

      E3= ( shat1 +qsq1 -qsq2 -qsq4 )/2.d0/sqrt( shat )
      if( E3.lt.0.d0 ) then
         ifail= 1
         return
      endif

      PT2lim= ( E2 +E3 )**2 -shat1
      if( ldebug.ge.2 ) print '(5X,''pt-sq(lim) '',E14.6)',PT2lim
      if( PT2lim.lt.0.d0 ) then
         if( ldebug.ge.1 ) print *,'!!! pt-sq(lim) < 0 ',
     >        '-> set to zero !!!'
         PT2lim= 0.d0
      endif
      if( PT2.gt.PT2lim ) then
         if( ldebug.ge.1 ) print *,'!!! pt-sq > pt-sq(lim) ',
     >        '-> forced to be pt-sq = pt-sq(lim) !!!'
         PT2= PT2lim
      endif
      DPL2= PT2lim -PT2
      if( DPL2.lt.0.d0 ) then
         if( ldebug.ge.1 ) print *,'!!! D_pl-sq < 0 ',
     >        '-> forced to be zero !!!'
         DPL2= 0.d0
      endif
      PL= P1 -sqrt( DPL2 )
      PT= sqrt( PT2 )

      if( ldebug.ge.2 ) print '(5X,''=> E3, PL, PT = '',3E14.6)', 
     >     E3,PL,PT

C  Randomize the phi direction.
      rmphi= ( QPRAND(idum)*2.d0 -1.d0 )*pi
      px= PT*cos( rmphi )
      py= PT*sin( rmphi )
      if( ldebug.ge.2 ) then
         print '(5X,''phi, px, py = '',3E14.6)', rmphi,px,py
         snew= ( E3 +E2 )**2 -px**2 -py**2 -( P1 -PL )**2
         print '(10X,''new s = '',E14.6)', snew
      endif

C  Boost back the "child".
      if( ldebug.ge.2 ) print *,'Boost-back the "child".'
      ptemp(1)= px
      ptemp(2)= py
      ptemp(3)= PL
      ptemp(4)= E3
      call bstbk( ptemp, bdir, gamma, phi, theta, PLPTN(4,ipchild) )
      PLPTN(3,ipchild)= PT**2

      if( ldebug.ge.2 ) then
         qsqdif= PLPTN(7,ipchild)**2 -PLPTN(4,ipchild)**2
     >        -PLPTN(5,ipchild)**2 -PLPTN(6,ipchild)**2 
     >        -PLPTN(2,ipchild)
         print '(5X,''q-sq. diff = '',E14.6)', qsqdif
      endif

C  Boost back the radiation.
      if( ldebug.ge.2 ) print *,'Boost-back the radiation.'
      ptemp(1)= -px
      ptemp(2)= -py
      ptemp(3)= P1 -PL
      ptemp(4)= E1 -E3
      call bstbk( ptemp, bdir, gamma, phi, theta, PLPTN(4,iprad) )
      PLPTN(3,iprad)= PT**2

      if( ldebug.ge.2 ) then
         qsqdif= PLPTN(7,iprad)**2 -PLPTN(4,iprad)**2
     >        -PLPTN(5,iprad)**2 -PLPTN(6,iprad)**2 
     >        -PLPTN(2,iprad)
         print '(5X,''q-sq. diff = '',E14.6)', qsqdif
      endif

      if( ldebug.ge.2 ) then
         print '(5X,''mother: '',4E14.6)', (PLPTN(k,ipmoth),k=4,7)
         print '(5X,''child:  '',4E14.6)', (PLPTN(k,ipchild),k=4,7)
         print '(5X,''rad:    '',4E14.6)', (PLPTN(k,iprad),k=4,7)
         print '(5X,''target: '',4E14.6)', (PLPTN(k,iptarget),k=4,7)
         snew= ( PLPTN(7,ipchild) +PLPTN(7,iptarget) )**2
     >        -( PLPTN(4,ipchild) +PLPTN(4,iptarget) )**2
     >        -( PLPTN(5,ipchild) +PLPTN(5,iptarget) )**2
     >        -( PLPTN(6,ipchild) +PLPTN(6,iptarget) )**2
         print '(10X,''new s = '',E14.6)', snew
      endif

C  Go to the next branch
      xpart(iside)= xpart(iside)*zbr
      ippart(iside)= ipchild
      goto 1

      END
*----+---1---------2---------3---------4---------5---------6---------7-+
      double precision function ptnmas( id )
C
C  Return the parton mass.
C  Only light 5 quarks and the gluon are allowed.
C
      implicit real*8(a-h,o-z)
      include 'inclm.h'

      ida= iabs( id )
      if( ida.eq.1 ) then
         ptnmas= amdq(1)
      elseif( ida.eq.2 ) then
         ptnmas= amuq(1)
      elseif( ida.eq.3 ) then
         ptnmas= amdq(2)
      elseif( ida.eq.4 ) then
         ptnmas= amuq(2)
      elseif( ida.eq.5 ) then
         ptnmas= amdq(3)
      elseif( ida.eq.6 ) then
         ptnmas= amuq(3)
      elseif( ida.eq.21 ) then
         ptnmas= amg
      else
         print *,'PTNMAS: abnormal parton ID. ',id
         stop
      endif

      return
      end
*----+---1---------2---------3---------4---------5---------6---------7-+
      subroutine bstcm( p1, p2,
     >     pcm, e1, e2, bdir, gamma, phi, theta )
C
C  Obtain the parameters to boost the input two-particle system to
C  their center-of-mass frame.
C  The momentum and the energies in the CM frame are returned.
C  The rotation angles to align the momenta along the z-axis are also 
C  obtained, where the 1st particle always directs to the positive side.
C  13 April 2005, S. Odaka (KEK)
C
C  (input)  p1(i): momentum and energy of the 1st particle
C           p2(i): momentum and energy of the 2nd particle
C
C  (output) pcm: momentum of the 1st particle (> 0) in the CM frame
C           e1: energy of the 1st particle in the CM frame
C           e2: energy of the 2nd particle in the CM frame
C           bdir(i): boost direction (unit vector)
C           gamma: gamma value of the Lorentz boost
C           phi: phi rotation angle to make py = 0; px(1) > 0 after
C                the rotation
C           theta: theta rotation angle to make px = 0; pz(1) > 0
C                  after the rotation
C
      implicit real*8(a-h,o-z)
      dimension p1(4),p2(4), bdir(3)
      dimension ptemp(3)

CCC      print *,'BSTCM'

C  Reset the outputs.
      pcm= 0.d0
      e1= 0.d0
      e2= 0.d0
      bdir(1)= 0.d0
      bdir(2)= 0.d0
      bdir(3)= 1.d0
      gamma= 1.d0
      phi= 0.d0
      theta= 0.d0

C  Lorentz-boost parameters.
      esys= p1(4) +p2(4)
      if( esys.lt.0.d0 ) then
         print *,'BSTCM: negative total energy.'
         print '(5X,''p1: '',4E12.4)',(p1(i),i=1,4)
         print '(5X,''p2: '',4E12.4)',(p2(i),i=1,4)
         stop
      endif
      pabs= 0.d0
      do 10 i=1,3
         ptemp(i)= p1(i) +p2(i)
         pabs= pabs +ptemp(i)**2
 10   continue
      s= esys**2 -pabs
      pabs= sqrt( pabs )
      if( s.lt.0.d0 ) then
         print *,'BSTCM: negative s.'
         stop
      endif
      roots= sqrt( s )
      gamma= esys/roots
      beta= sqrt( 1.d0 -1.d0/gamma**2 )
      if( beta.gt.1.d-10 ) then
         do 20 i=1,3
            bdir(i)= ptemp(i)/pabs
 20      continue
CCC      else
CCC         print '('' BSTCM: very small beta.'',E14.6)', beta
      endif

C  Boost the 1st particle to obtain the energy-momentum and rotation 
C  angles.
C  Lorentz boost.
      plin= p1(1)*bdir(1) +p1(2)*bdir(2) +p1(3)*bdir(3)
      do 30 i=1,3
         ptemp(i)= p1(i) -plin*bdir(i)
 30   continue
      ein1= p1(4)
      e1= gamma*( ein1 -beta*plin )
      pl= gamma*( -beta*ein1 +plin )
      do 40 i=1,3
         ptemp(i)= ptemp(i) +pl*bdir(i)
 40   continue
      pcm= sqrt( ptemp(1)**2 +ptemp(2)**2 +ptemp(3)**2 )
CCC      print '(5X,''p(cm) = '',E14.6)', pcm
C  Rotation.
      phi= atan2( ptemp(2), ptemp(1) )
      px= ptemp(1)*cos( phi ) +ptemp(2)*sin( phi )
      theta= atan2( px, ptemp(3) )
CCC      pz= ptemp(3)*cos( theta ) +px*sin( theta )
CCC      print '(5X,''pz (final) = '',E14.6)', pz

C  2nd particle for test.
CCC      plin= p2(1)*bdir(1) +p2(2)*bdir(2) +p2(3)*bdir(3)
CCC      ein2= p2(4)
CCC      e2= gamma*( ein2 -beta*plin )
CCC      print '(5X,''E2 from boost = '',E14.6)', e2

C  The energy of the 2nd particle from root(s).
      e2= roots -e1
CCC      print '(5X,''E2 from root(s) = '',E14.6)', e2

      return
      end

*----+---1---------2---------3---------4---------5---------6---------7-+
      subroutine bstbk( pin, bdir, gamma, phi, theta, pbk )
C
C  Boost back a momentum to the laboratory frame using the rotation and 
C  Lorentz boost parameters determined by the subroutine BSTCM.
C  13 April 2005, S. Odaka (KEK)
C
C  (input)  pin(i): input momentum and energy of the particle
C           bdir(i): direction of the boost (unit vector)
C           gamma: gamma value of the Lorentz boost
C           phi: phi rotation angle
C           theta: theta rotation angle
C
C  (output) pbk(i): momentum and energy boosted back
C
      implicit real*8(a-h,o-z)
      dimension pin(4),pbk(4),bdir(3)

C  Theta rotation.
      sn= sin( theta )
      cs= cos( theta )
      px= pin(1)*cs +pin(3)*sn
      pz= -pin(1)*sn +pin(3)*cs
      py= pin(2)

C  Phi rotation.
      sn= sin( phi )
      cs= cos( phi )
      pbk(1)= px*cs -py*sn
      pbk(2)= px*sn +py*cs
      pbk(3)= pz

C  Lorentz boost.
      beta= sqrt( 1.d0 -1.d0/gamma**2 )
      plin= pbk(1)*bdir(1) +pbk(2)*bdir(2) +pbk(3)*bdir(3)
      do 10 i=1,3
         pbk(i)= pbk(i) -plin*bdir(i)
 10   continue
      plbk= gamma*( beta*pin(4) +plin )
      do 20 i=1,3
         pbk(i)= pbk(i) +plbk*bdir(i)
 20   continue
      pbk(4)= gamma*( pin(4) +beta*plin )

      return
      end

*----+---1---------2---------3---------4---------5---------6---------7-+
      subroutine bstfw( pin, bdir, gamma, phi, theta, pfw )
C
C  Boost the input momentum to the CM frame using the rotation and 
C  Lorentz boost parameters determined by the subroutine BSTCM.
C  25 August 2005, S. Odaka (KEK)
C
C  (input)  pin(i): input momentum and energy of the particle
C           bdir(i): direction of the boost (unit vector)
C           gamma: gamma value of the Lorentz boost
C           phi: phi rotation angle
C           theta: theta rotation angle
C
C  (output) pfw(i): momentum and energy in the CM frame
C
      implicit real*8(a-h,o-z)
      dimension pin(4),pfw(4),bdir(3)

C  Lorentz boost.
      beta= sqrt( 1.d0 -1.d0/gamma**2 )
      plin= pin(1)*bdir(1) +pin(2)*bdir(2) +pin(3)*bdir(3)
      do 10 i=1,3
         pfw(i)= pin(i) -plin*bdir(i)
 10   continue
      plfw= gamma*( -beta*pin(4) +plin )
      do 20 i=1,3
         pfw(i)= pfw(i) +plfw*bdir(i)
 20   continue
      pfw(4)= gamma*( pin(4) -beta*plin )

C  Phi rotation
      px= pfw(1)
      py= pfw(2)
      csph= cos( phi )
      snph= sin( phi )
      pfw(1)= px*csph +py*snph
      pfw(2)= -px*snph +py*csph

C  Theta rotation
      px= pfw(1)
      pz= pfw(3)
      csth= cos( theta )
      snth= sin( theta )
      pfw(1)= px*csth -pz*snth
      pfw(3)= px*snth +pz*csth

      return
      end
